
#include <stdlib.h>
#include <string.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "mpu6050/mpu6050.h"

#include "DataTypes.h"
#include "DefaultFonts.h"
#include "UTFT.h"

#define WIDTH 5

#define LIMIT 0.2

#define BORDER_MIN_X 14
#define BORDER_MIN_Y 54
#define BORDER_MAX_X 310
#define BORDER_MAX_Y 230

#define LEFT 1
#define RIGHT 2
#define UP 3
#define DOWN 4

#define START_X 10
#define START_Y 90

#define SCREEN_LEFT_X 0
#define SCREEN_LEFT_Y 0
#define SCREEN_RIGHT_X 319
#define SCREEN_RIGHT_Y 239

#define ITEM_COLOR_R 70
#define ITEM_COLOR_G 90
#define ITEM_COLOR_B 18

#define TOP_LINE_X1 SCREEN_LEFT_X + 2
#define TOP_LINE_Y1 20
#define TOP_LINE_X2 SCREEN_RIGHT_X - 2
#define TOP_LINE_Y2 20

#define RECT_LEFT_X SCREEN_LEFT_X + 2
#define RECT_LEFT_Y 25
#define RECT_RIGHT_X SCREEN_RIGHT_X - 2
#define RECT_RIGHT_Y SCREEN_RIGHT_Y - 2

#define SCORE_X TOP_LINE_X1
#define SCORE_Y TOP_LINE_Y1 - 15

using namespace std;

extern const unsigned char SmallFont[];

typedef struct {
	int x;
	int y;
} pixel;

UTFT myGLCD;

pixel snake[100];
pixel food;
char dir = RIGHT, prev_dir = RIGHT;
int size = 1;
bool eat = false, lose = false;

void move_snake() {
	pixel last_cell;

	last_cell.x = snake[size - 1].x;
	last_cell.y = snake[size - 1].y;

	for (int i = size - 1; i >= 1; i--) {
		snake[i].x = snake[i-1].x;
		snake[i].y = snake[i-1].y;
	}

	switch (dir) {
	    case UP: {
			if (prev_dir == DOWN) {
				snake[0].y += WIDTH;
				dir = DOWN;
			}
			else {
				snake[0].y -= WIDTH;
			}

			break;
		}
		case DOWN: {
			if (prev_dir == UP) {
				snake[0].y -= WIDTH;
				dir = UP;
			}
			else {
				snake[0].y += WIDTH;
			}
			break;
		}
		case RIGHT: {
			if (prev_dir == LEFT) {
				snake[0].x -= WIDTH;
				dir = LEFT;
			}
			else {
				snake[0].x += WIDTH;
			}

			break;
		}
		case LEFT: {
			if (prev_dir == RIGHT) {
				snake[0].x += WIDTH;
				dir = RIGHT;
			}
			else {
				snake[0].x -= WIDTH;
			}

			break;
		}
	}

    if (snake[0].x == food.x && snake[0].y == food.y) {
        size++;

        snake[size - 1].x = last_cell.x;
        snake[size - 1].y = last_cell.y;

        eat = true;
	}

	if (snake[0].x <= RECT_LEFT_X) {
        snake[0].x = RECT_RIGHT_X - 1;
	}
	else if (snake[0].x >= RECT_RIGHT_X) {
        snake[0].x = RECT_LEFT_X + 1;
	}

	if (snake[0].y <= RECT_LEFT_Y) {
        snake[0].y = RECT_RIGHT_Y - 10;
	}
	else if (snake[0].y >= RECT_RIGHT_Y - 3) {
        snake[0].y = RECT_LEFT_Y + 1;
	}
}

int main(void) {
    double axg = 0;
    double ayg = 0;
    double azg = 0;
    double gxds = 0;
    double gyds = 0;
    double gzds = 0;

    char score[15];
    strcpy(score, "Score: ");

	// Init accelerometer.
	mpu6050_init();

	// Init LCD.
    myGLCD.InitLCD();
  	myGLCD.setFont(SmallFont);
  	myGLCD.clrScr();

  	// Classical Nokia Snake background color.
    myGLCD.fillScr(50, 200, 11);

    myGLCD.setColor(ITEM_COLOR_R, ITEM_COLOR_G, ITEM_COLOR_B);

    myGLCD.drawLine(TOP_LINE_X1, TOP_LINE_Y1, TOP_LINE_X2, TOP_LINE_Y2);
    myGLCD.drawRect(RECT_LEFT_X, RECT_LEFT_Y, RECT_RIGHT_X, RECT_RIGHT_Y);

    myGLCD.setColor(ITEM_COLOR_R, ITEM_COLOR_G, ITEM_COLOR_B);

  	snake[0].x = START_X;
  	snake[0].y = START_Y;

  	food.x = START_X + 100;
  	food.y = START_Y;

    myGLCD.fillRect(food.x, food.y, food.x + WIDTH, food.y + WIDTH);

	while(1) {
		mpu6050_getConvData(&axg, &ayg, &azg, &gxds, &gyds, &gzds);

		myGLCD.setColor(50, 200, 11);

        myGLCD.fillRect(snake[size - 1].x, snake[size - 1].y, snake[size - 1].x + WIDTH, snake[size - 1].y + WIDTH);

		if(axg > LIMIT) {
            dir = DOWN;
		}
		else if(axg < -LIMIT) {
            dir = UP;
		}

		if(ayg > LIMIT) {
            dir = RIGHT;
		}
		else if(ayg < -LIMIT) {
            dir = LEFT;
		}

        move_snake();

        myGLCD.setColor(ITEM_COLOR_R, ITEM_COLOR_G, ITEM_COLOR_B);

        myGLCD.fillRect(snake[0].x, snake[0].y, snake[0].x + WIDTH, snake[0].y + WIDTH);

        for (int i = 1; i < size; i++) {
            if(snake[0].x == snake[i].x && snake[0].y == snake[i].y) {
                lose = true;
            }

            myGLCD.fillRect(snake[i].x, snake[i].y, snake[i].x + WIDTH, snake[i].y + WIDTH);
        }

        // Put new food.
        if(eat) {
            bool ok = false;

            while (!ok) {
                 food.x = rand() % (RECT_RIGHT_X - RECT_LEFT_X);

                if (food.x % WIDTH == 0) {
                    ok = true;
                }
            }

            ok = false;

            while (!ok) {
                food.y = rand() % (RECT_RIGHT_Y - RECT_LEFT_Y);

                if (food.y % WIDTH == 0) {
                    ok = true;
                }
            }

            myGLCD.setColor(ITEM_COLOR_R, ITEM_COLOR_G, ITEM_COLOR_B);

            myGLCD.fillRect(food.x, food.y, food.x + WIDTH, food.y + WIDTH);

            eat = false;
        }

        // Print the score.
        myGLCD.setColor(255, 0 , 0);
        itoa(size - 1, score + 7, 10);
        myGLCD.print(score, SCORE_X, SCORE_Y);

        if(lose == true)
            break;

        prev_dir = dir;

		_delay_ms(100);
	}

	myGLCD.print("GAME OVER!", 150, START_Y);
 }
